<?php

namespace Drp\Analytics\Foundation;

use Drp\Analytics\Contracts\HandlesAnalytics;
use Drp\Analytics\Foundation\Handlers\EloquentAnalytic;
use Drp\Analytics\Foundation\Handlers\EloquentMetric;
use Drp\Analytics\Jobs\StoreAnalytic;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Support\Arr;
use Illuminate\Support\Traits\Macroable;

class AnalyticsManager
{
    use Macroable;

    public const DRIVER_ELOQUENT = 'eloquent';

    public $handlers = [
        'analytic' => [
            self::DRIVER_ELOQUENT => EloquentAnalytic::class,
        ],
        'metric' => [
            self::DRIVER_ELOQUENT => EloquentMetric::class,
        ],
    ];

    /**
     * @param mixed|null $payload
     */
    public function store(string $event, $payload = null): void
    {
        $reference = new AnalyticReference(AnalyticableCookie::pull(), $event);
        $payload = new AnalyticPayload($payload);

        StoreAnalytic::dispatch($this->handlerFor($event), $reference, $payload)
            ->onQueue(config('analytics.queue'));
    }

    public function get(string $event, ?callable $query = null): Collection
    {
        return $this->handlerFor($event)->get($event, $query);
    }

    private function handlerFor(string $event): HandlesAnalytics
    {
        $concept = config("analytics.trackers.{$event}") ?? config('analytics.undefined-tracker-concept');
        $driver = config("analytics.drivers.{$concept}");

        $handler = Arr::get($this->handlers, "{$concept}.{$driver}");

        return new $handler;
    }

    public function registerHandler(string $concept, string $driver, string $handler): void
    {
        Arr::set($this->handlers, "{$concept}.{$driver}", $handler);
    }

    public function renderJs(): string
    {
        return view('Analytics::analytics-js')->render();
    }
}
