<?php

namespace Drp\Analytics\Models;

use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;

class Analytic extends Model
{
    protected $fillable = [
        'event',
        'analyticable_id',
        'analyticable_type',
        'payload',
    ];

    protected $casts = [
        'payload' => 'array',
    ];

    public function scopeByType(Builder $query, string $analyticableType): Builder
    {
        return $query->where('analyticable_type', $analyticableType);
    }

    public static function eventCount(string $event, ?string $analyticableType = null, bool $distinct = false): int
    {
        return self::query()
            ->select('analyticable_id')
            ->whereEvent($event)
            ->when($analyticableType, function (Builder $query) use ($analyticableType): Builder {
                return $query->byType($analyticableType);
            })
            ->when($distinct, function (Builder $query): Builder {
                return $query->distinct('analyticable_id');
            })
            ->count();
    }
}
